#include "IZ_wsclient.h"

void IZ_WSClientAttemptConnect(struct lws_sorted_usec_list *sul) {
	IZ_WSClientVHostData* vhd = lws_container_of(sul, IZ_WSClientVHostData, sul);

	printf("%s %s %u", vhd->address, vhd->path, *vhd->port);

	vhd->i.context = vhd->context;
	vhd->i.port = *vhd->port;
	vhd->i.address = vhd->address;
	vhd->i.path = vhd->path;
	vhd->i.host = vhd->i.address;
	vhd->i.origin = vhd->i.address;
	vhd->i.ssl_connection = 0;

	vhd->i.protocol = NETWORK_PROTOCOL;
	vhd->i.pwsi = &vhd->client_wsi;

	if (lws_client_connect_via_info(&vhd->i)) {
		return;
	}

	lws_sul_schedule(
		vhd->context,
		0,
		&vhd->sul,
		IZ_WSClientAttemptConnect,
		10 * LWS_US_PER_SEC
	);
}

IZ_ProcedureResult IZ_WSClientProtocolInitialize(struct lws* wsi, void* in) {
	IZ_WSClientVHostData* vhd_instance = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);
	IZ_WSClientVHostData** vhd = &vhd_instance;
	*vhd = lws_protocol_vh_priv_zalloc(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi),
		sizeof(IZ_WSClientVHostData)
	);
	(*vhd)->context = lws_get_context(wsi);
	(*vhd)->protocol = lws_get_protocol(wsi);
	(*vhd)->vhost = lws_get_vhost(wsi);
	(*vhd)->port = (u16*) lws_pvo_search(
		(const struct lws_protocol_vhost_options *)in,
		"port")->value;
	(*vhd)->address = lws_pvo_search(
		(const struct lws_protocol_vhost_options *)in,
		"address")->value;
	(*vhd)->path = lws_pvo_search(
		(const struct lws_protocol_vhost_options *)in,
		"path")->value;
	(*vhd)->ring = lws_ring_create(
		sizeof(IZ_WebsocketMessage),
		RING_COUNT,
		IZ_WebsocketDestroyMessage
	);
	if (!(*vhd)->ring) {
		return 1;
	}
	IZ_WSClientAttemptConnect(&(*vhd)->sul);
	return 0;
}

void IZ_WSClientProtocolTeardown(struct lws* wsi) {
	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);

	vhd->finished = true;
	if (vhd->ring) {
		lws_ring_destroy(vhd->ring);
	}

	lws_sul_cancel(&vhd->sul);
}

void IZ_WSClientConnectionError(struct lws* wsi, void* in) {
	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);

	lwsl_err("CLIENT_CONNECTION_ERROR: %s\n", in ? (char *)in : "(null)");
	vhd->client_wsi = NULL;
	lws_sul_schedule(
		vhd->context,
		0,
		&vhd->sul,
		IZ_WSClientAttemptConnect,
		LWS_US_PER_SEC
	);
}

void IZ_WSClientOnOpen(struct lws* wsi) {
	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);

	vhd->established = true;
}

IZ_ProcedureResult IZ_WSClientWritable(struct lws* wsi) {
	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);

	const IZ_WebsocketMessage* pmsg = lws_ring_get_element(vhd->ring, &vhd->tail);
	if (!pmsg) {
		return 0;
	}

	/* notice we allowed for LWS_PRE in the payload already */
	i32 m = lws_write(
		wsi,
		((unsigned char*) pmsg->payload) + LWS_PRE,
		pmsg->len,
		pmsg->binary ? LWS_WRITE_BINARY : LWS_WRITE_TEXT
	);

	if (m < (i32)pmsg->len) {
		lwsl_err("ERROR %d writing to ws socket\n", m);
		return -1;
	}

	lws_ring_consume_single_tail(vhd->ring, &vhd->tail, 1);

	/* more to do for us? */
	if (lws_ring_get_element(vhd->ring, &vhd->tail)) {
		/* come back as soon as we can write more */
		lws_callback_on_writable(wsi);
	}

	return 0;
}

void IZ_WSClientOnClose(struct lws* wsi) {
	IZ_WSClientVHostData* vhd = (IZ_WSClientVHostData*) lws_protocol_vh_priv_get(
		lws_get_vhost(wsi),
		lws_get_protocol(wsi)
	);

	vhd->client_wsi = NULL;
	vhd->established = false;
	lws_sul_schedule(
		vhd->context,
		0,
		&vhd->sul,
		IZ_WSClientAttemptConnect,
		LWS_US_PER_SEC
	);
}

void IZ_WSClientOnReceive(struct lws* wsi, IZ_WSClientSessionData* pss, void* in, size_t len) {
	lwsl_user("LWS_CALLBACK_CLIENT_RECEIVE: %4d (rpp %5d, first %d, last %d, bin %d)\n",
						(int)len, (int)lws_remaining_packet_payload(wsi),
						lws_is_first_fragment(wsi),
						lws_is_final_fragment(wsi),
						lws_frame_is_binary(wsi));

	// lwsl_hexdump_notice(in, len);
	IZ_WebsocketMessage amsg;
	amsg.first = (char)lws_is_first_fragment(wsi);
	amsg.final = (char)lws_is_final_fragment(wsi);
	amsg.binary = (char)lws_frame_is_binary(wsi);
	i32 n = (int)lws_ring_get_count_free_elements(pss->ring);
	if (!n) {
		lwsl_user("dropping!\n");
		return;
	}

	amsg.len = len;
	/* notice we over-allocate by LWS_PRE */
	amsg.payload = malloc(LWS_PRE + len);
	if (!amsg.payload) {
		lwsl_user("OOM: dropping\n");
		return;
	}

	memcpy((char *)amsg.payload + LWS_PRE, in, len);
	if (!lws_ring_insert(pss->ring, &amsg, 1)) {
		IZ_WebsocketDestroyMessage(&amsg);
		lwsl_user("dropping!\n");
		return;
	}
	lws_callback_on_writable(wsi);

	if (!pss->flow_controlled && n < 3) {
		pss->flow_controlled = 1;
		lws_rx_flow_control(wsi, 0);
	}
}

IZ_ProcedureResult IZ_WSClientCallback(
	struct lws* wsi,
	enum lws_callback_reasons reason,
	void* user,
	void* in,
	size_t len
) {
	switch (reason) {
		case LWS_CALLBACK_PROTOCOL_INIT:
			return IZ_WSClientProtocolInitialize(wsi, in);
		case LWS_CALLBACK_PROTOCOL_DESTROY:
			IZ_WSClientProtocolTeardown(wsi);
			break;
		case LWS_CALLBACK_CLIENT_CONNECTION_ERROR:
			IZ_WSClientConnectionError(wsi, in);
			break;
		case LWS_CALLBACK_CLIENT_ESTABLISHED:
			IZ_WSClientOnOpen(wsi);
			break;
		case LWS_CALLBACK_CLIENT_CLOSED:
			IZ_WSClientOnClose(wsi);
			break;
		case LWS_CALLBACK_CLIENT_RECEIVE:
			IZ_WSClientOnReceive(wsi, user, in, len);
			break;
		case LWS_CALLBACK_CLIENT_WRITEABLE:
			return IZ_WSClientWritable(wsi);
		default:
			break;
	}

	return lws_callback_http_dummy(wsi, reason, user, in, len);
}

IZ_ProcedureResult IZ_WSClientInitialize(IZ_WSClientState* state, IZ_WSClientInitializeParams params) {
	struct lws_context_creation_info info;
	memset(&info, 0, sizeof info);
	info.port = CONTEXT_PORT_NO_LISTEN;

	static const struct lws_protocols protocols[] = {
		{
			.name = NETWORK_PROTOCOL,
			.callback = IZ_WSClientCallback,
			.per_session_data_size = sizeof(IZ_WSClientSessionData),
			.rx_buffer_size = 1024,
			.id = 0,
			.user = NULL,
			.tx_packet_size = 0,
		},
		LWS_PROTOCOL_LIST_TERM
	};
	info.protocols = protocols;

	static struct lws_protocol_vhost_options pvo_address = {
		NULL,
		NULL,
		"address",		/* pvo name */
		"localhost"	/* pvo value */
	};
	pvo_address.value = params.address;

	static struct lws_protocol_vhost_options pvo_path = {
		&pvo_address,
		NULL,
		"path",		/* pvo name */
		"/"	/* pvo value */
	};
	pvo_path.value = params.path;

	static struct lws_protocol_vhost_options pvo_port = {
		&pvo_path,
		NULL,
		"port",		/* pvo name */
		NULL	/* pvo value */
	};
	pvo_port.value = (void*) &params.port;

	static const struct lws_protocol_vhost_options pvo = {
		NULL,		/* "next" pvo linked-list */
		&pvo_port,	/* "child" pvo linked-list */
		NETWORK_PROTOCOL,	/* protocol name we belong to on this vhost */
		""		/* ignored */
	};

	info.pvo = &pvo;
	info.fd_limit_per_thread = 1 + 1 + 1;
	IZ_WebsocketInitialize(&state->ws);
	state->ws.context = lws_create_context(&info);
	if (!state->ws.context) {
		return -1;
	}
	return 0;
}

IZ_ProcedureResult IZ_WSClientHandle(IZ_WSClientState* state) {
	return IZ_WebsocketHandle(&state->ws);
}

void IZ_WSClientTeardown(IZ_WSClientState* state) {
	IZ_WebsocketTeardown(&state->ws);
}

void IZ_WSClientCancelService(IZ_WSClientState* state) {
	IZ_WebsocketCancelService(&state->ws);
}
