#ifndef IZ_WSSERVER_H
#define IZ_WSSERVER_H

#include "libwebsockets.h"
#include <string.h>
#include "../IZ_common.h"
#include "IZ_websocket.h"

/* one of these is created for each client connecting to us */
typedef struct IZ_WSServerSessionData {
	struct IZ_WSServerSessionData* pss_list;
	struct lws* wsi;
	u32 tail;
	u8 culled: 1;
} IZ_WSServerSessionData;

/* one of these is created for each vhost our protocol is used with */
typedef struct {
	struct lws_context *context;
	struct lws_vhost *vhost;
	const struct lws_protocols *protocol;
	IZ_WSServerSessionData *pss_list; /* linked-list of live pss*/
	struct lws_ring *ring; /* ringbuffer holding unsent messages */
} IZ_WSServerVHostData;

IZ_ProcedureResult IZ_WSServerCallback(
	struct lws*,
	enum lws_callback_reasons,
	void*,
	void*,
	size_t
);

typedef struct {
	u16 port;
	const char* origin;
	const char* default_filename;
	const char* protocol;
} IZ_WSServerInitializeParams;

typedef struct {
	i32 log_level;
	u16 port;
} IZ_WSServerConfig;

typedef struct {
	IZ_WSServerConfig config;
	IZ_Websocket ws;
} IZ_WSServerState;

IZ_ProcedureResult IZ_WSServerInitialize(IZ_WSServerState*, IZ_WSServerInitializeParams);

IZ_ProcedureResult IZ_WSServerHandle(IZ_WSServerState*);

void IZ_WSServerTeardown(IZ_WSServerState*);

void IZ_WSServerCancelService(IZ_WSServerState*);

#endif
